use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use XML::Simple;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject);
use DE_EPAGES::Amazon::API::RemoteAdapterFactory;

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $NoOutput;
    my $IDs;
    my $AmazonPlatformAlias;
    my $ShopAlias;


    GetOptions(
        'platform=s'  => \$AmazonPlatformAlias,
        'shop=s'      => \$ShopAlias,
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'ids=s'       => \$IDs,
    );

    usage() if $Help;
    usage("Missing parameter: storename") unless $StoreName;
    usage("Missing parameter: shop") unless $ShopAlias;
    usage("Missing parameter: platform") unless $AmazonPlatformAlias;
    usage("Missing parameter: ids") unless $IDs;
    
    my $time1 = time;
    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { GetMatchingProducts( ($NoOutput) ? 0 : 1, $StoreName, $AmazonPlatformAlias, $ShopAlias, $IDs ) },
    );

    my $logmsg = sprintf 'need %.1f seconds', time() - $time1;
    print $logmsg. "\n" unless $NoOutput;
    GetLog()->debug("Amazon GetMatchingProducts: $logmsg");

}

sub GetMatchingProducts {
    my ( $printOutput, $StoreName, $AmazonPlatformAlias, $ShopAlias, $IDs ) = @_;
    # make sure that the cartridge is installed
    unless ( ExistsCartridgeByPackage('DE_EPAGES::Amazon') ) {
        print 'Store: ' . $StoreName . ": Cartridge Amazon is not installed.\n" if ($printOutput);
        return;
    }

    my $ShopFolder = LoadRootObject()->child('Shops');
    usage("Shop $ShopAlias not found.") unless($ShopFolder->existsChild($ShopAlias));


    my $UsedPortalConfig;
    my $Shop = $ShopFolder->child($ShopAlias);
    foreach my $PortalConfig (@{$Shop->get('PortalConfigs')}){
        next unless ($PortalConfig->instanceOf('AmazonConfig'));
        next unless ($PortalConfig->get('PortalSite')->alias eq $AmazonPlatformAlias);
        $UsedPortalConfig = $PortalConfig;
    }

    usage("Portal not found") unless($UsedPortalConfig);
   
    # load portal settings and call GetAuthToken
    my $Remote = DE_EPAGES::Amazon::API::RemoteAdapterFactory::NewRemoteMWSProduct( $UsedPortalConfig );
    my $aIDs = [split /\s+/, $IDs];

    my $ahProductsForIDs = $Remote->GetMatchingProductForId( 'EANs' => $aIDs, 'IdType' => 'EAN' );
 
    # filter ASINs
    my $ASINMatches; 
    foreach my $ResponseElement (@$ahProductsForIDs){

            $ASINMatches->{$ResponseElement->{'Id'}} = [];
            next unless $ResponseElement->{'status'} eq 'Success';
            my $aProducts = $ResponseElement->{'Products'}->{'Product'};

            # we need an array also if we have only one element (JSON problem)
            $aProducts = ref $aProducts eq 'ARRAY' ? $aProducts : [$aProducts];

            foreach my $hProduct (@$aProducts){
                push @{$ASINMatches->{$ResponseElement->{'Id'}}}, $hProduct->{'Identifiers'}->{'MarketplaceASIN'}->{'ASIN'}
         
            }
    }

    foreach my $Key (keys(%$ASINMatches)) {
        print "ASINS found for EAN: $Key => ASIN(s):" . join(' and ', @{$ASINMatches->{$Key}}) ."\n";
    }

}


sub usage {
    my ($Message) = @_;

    print <<END_USAGE;

$Message


Description:
Uses GetMatchingProductForId call for fetching ASIN(s) from Amazon by given EAN(s).

Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -shop         name of shop
    -platform     Amazon platform                                           
    -ids          quoted list of eans like "978-3-314-10343-8 4011800574215" 

flags:

    -help       show the command line options

Example:
    perl $0 -storename Store -shop DemoShop -platform Amazon_de_DE -ids "4040729000772 978-3-314-10343-8 4011800574215 4003692210555 8690842346248 4004633292357 6417084182692 4015493880431"

END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
