#========================================================================================
# package      DE_EPAGES::AppStore::t::unit::UI_Shop
# base         DE_EPAGES::Test::API::UnitTest::Test
# state        private
# description  unit test for the Shop UI
#========================================================================================
package DE_EPAGES::AppStore::t::unit::UI_Shop;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall);
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Exception;
use Test::More;
 
use strict;

sub test {
        my $self = shift;

        $self->when_saving_without_selected_scopes_then_form_error();
        $self->when_saving_with_selected_scopes_then_no_form_error();
        $self->when_saving_with_selected_scopes_and_invalid_callback_url_then_form_error();
        $self->when_saving_with_selected_scopes_and_valid_callback_url_then_no_form_error();

        done_testing();
}

#----------------------------------------------------------------------------------------
sub when_saving_without_selected_scopes_then_form_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hFormValuesEmpty = {};
    my $Form = $self->_createForm($hFormValuesEmpty);
    my $MockedServlet = $self->_createServlet($Form);
    my $DeveloperAppsClient = $self->_createDeveloperAppsClient();
    my $Sut = $self->_createSut($DeveloperAppsClient);

    throws_ok( sub { $Sut->SaveDeveloperApp($MockedServlet) },
               qr/INVALID_FORM: the input field Scopes is not correct \(EMPTY_SCOPES\)/,
               "$SubTestName - proves that a missing scope throws a form error."
     );

}

#----------------------------------------------------------------------------------------
sub when_saving_with_selected_scopes_then_no_form_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hFormValues = {
        'AppName' => 'TestApp',
        'coupons_read' => 1
    };

    my $Form = $self->_createForm($hFormValues);
    my $MockedServlet = $self->_createServlet($Form);
    my $DeveloperAppsClient = $self->_createDeveloperAppsClient();
    my $Sut = $self->_createSut($DeveloperAppsClient);

    $Sut->SaveDeveloperApp($MockedServlet);

    my $aParameters = GetParametersFromMockifyCall($DeveloperAppsClient, 'createApp');
    my $aExpectedParameters = [ 'TestApp', undef, ['coupons_read'] ];
    is_deeply($aParameters, $aExpectedParameters, "$SubTestName - check that createApp is called with the selected scope.");
}

#----------------------------------------------------------------------------------------
sub when_saving_with_selected_scopes_and_invalid_callback_url_then_form_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hFormValues = {
            'coupons_read' => 1,
            'CallbackUrl' => 'not_a_valid_url'
    };
    my $Form = $self->_createForm($hFormValues);
    my $MockedServlet = $self->_createServlet($Form);
    my $DeveloperAppsClient = $self->_createDeveloperAppsClient();
    my $Sut = $self->_createSut($DeveloperAppsClient);

    throws_ok( sub { $Sut->SaveDeveloperApp($MockedServlet) },
               qr/INVALID_FORM: the input field CallbackUrl is not correct \(INVALID_URL\)/,
               "$SubTestName - proves that a not valid url throws a form error."
    );

}

#----------------------------------------------------------------------------------------
sub when_saving_with_selected_scopes_and_valid_callback_url_then_no_form_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hFormValues = {
            'AppName' => 'TestApp',
            'coupons_write' => 1,
            'CallbackUrl' => 'http://www.epages.com'
    };
    my $Form = $self->_createForm($hFormValues);
    my $MockedServlet = $self->_createServlet($Form);
    my $DeveloperAppsClient = $self->_createDeveloperAppsClient();
    my $Sut = $self->_createSut($DeveloperAppsClient);

    $Sut->SaveDeveloperApp($MockedServlet);

    my $aParameters = GetParametersFromMockifyCall($DeveloperAppsClient, 'createApp');
    my $aExpectedParameters = [ 'TestApp', 'http://www.epages.com', ['coupons_write'] ];
    is_deeply($aParameters, $aExpectedParameters, "$SubTestName - check that createApp is called with scopes and valid callback url.");

}

#----------------------------------------------------------------------------------------
#  helpers
#----------------------------------------------------------------------------------------
sub _createSut {
    my $self = shift;
    my ($DeveloperAppsClient) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::AppStore::UI::Shop',
                   $aParameterList
              );
    $Mockify->mock('_createDeveloperAppsClient', $DeveloperAppsClient, [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($Form) = @_;

    my $Shop = $self->_createShop();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $Shop );
    $MockifyServlet->addMockWithReturnValue('form', $Form);

    return $MockifyServlet->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {};
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;
    my ($hFormValues) = @_;

    my $aMockedParameters = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Presentation::API::Form', $aMockedParameters);

    my $aFormParameterChecks = [{'object'=> 'DE_EPAGES::Shop::API::Object::Shop'}, {'string'=>'DeveloperApp'}];
    $Mockify->mock('form', $hFormValues, $aFormParameterChecks);

    return $Mockify->getMockObject();
} 

#----------------------------------------------------------------------------------------
sub _createDeveloperAppsClient {
    my $self = shift;

    # The constructor of DeveloperAppsClient have two mandatory parameters
    # but, they are not needed for this test.
    my $hAttributes = [
        'AppsUriTemplate' => bless({},'dummyObject'),
        'AppUriTemplate' => bless({},'dummyObject2'),
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AppStore::API::Object::DeveloperAppsClient',
        $hAttributes
    );
    $Mockify->mock('createApp', 'dummyValue', ['string' ,'string', 'arrayref']);
    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();