#========================================================================================
# package      DE_EPAGES::Core::API::Scheduler
# state        public
# description  Functions for Scheduler.conf manipulation
#========================================================================================
package DE_EPAGES::Core::API::Scheduler;
use base qw (Exporter);
use strict;

use Config::IniFiles;
use Sys::Hostname;
use Clone qw(clone);

our @EXPORT_OK  = qw (
    GetMailParameters
);

#========================================================================================
# function     GetMailParameters
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MailParameters = GetMailParameters($Options);
# example      my $MailParameters = GetMailParameters(\%Options);
#----------------------------------------------------------------------------------------
# description  Returns a hash with merged mail parameters from $Options (higher precedence)
#               and Scheduler.conf (lower precedence).
#----------------------------------------------------------------------------------------
# input        $Options | options keys:
#               <ul>
#                 <li>ini      - path to Scheduler.conf
#                 <li>section  - section in Scheduler.conf
#                 <li>hostname - hostname to use
#                 <li>store    - store to use
#                 <li>from     - e-mail address
#                 <li>to       - list of addressees, separated by [,;]
#                 <li>cc       - list of addressees, separated by [,;]
#                 <li>bcc      - list of addressees, separated by [,;]
#                 <li>subject  - e-mail subject
#                 <li>content  - e-mail body
#                 <li>file     - content of file is e-mail body
#                 <li>descr    - additional description
#                 <li>attachments - attachment files
#                 <li>SMTPServer
#                 <li>SMTPPort
#                 <li>SMTPLogin
#                 <li>SMTPPassword
#               </ul> | ref.hash
# return       $MailParameters | hash with mail parameters - {in Scheduler.conf}:
#               <ul>
#                 <li>store    - store to use - {MailStore}
#                 <li>from     - e-mail address - {MailSender}
#                 <li>to       - list of addressees, separated by [,;] - {MailRecipient}
#                 <li>cc       - list of addressees, separated by [,;] {not in Scheduler.conf}
#                 <li>bcc      - list of addressees, separated by [,;] {not in Scheduler.conf}
#                 <li>subject  - e-mail subject - {not in Scheduler.conf}
#                 <li>content  - e-mail body - {not in Scheduler.conf}
#                 <li>file     - content of file is e-mail body - {not in Scheduler.conf}
#                 <li>descr    - additional description - {not in Scheduler.conf}
#                 <li>attachments  - attachment files - {not in Scheduler.conf}
#                 <li>SMTPServer   - {SMTPServer}
#                 <li>SMTPPort     - {SMTPPort}
#                 <li>SMTPLogin    - {SMTPLogin}
#                 <li>SMTPPassword - {SMTPPassword}
#               </ul> | ref.hash
#========================================================================================
sub GetMailParameters {
    my ($Options) = @_;

    my $Params = clone($Options);
    delete $Params->{$_} foreach qw(hostname section ini);

    my $Host = hostname();
    chomp($Host);
    $Host =~ s/\..*//;
    $Host = $Options->{hostname} if $Options->{hostname};
    my $Section = $Options->{section} // 'LOOPS';

    my %Ini;
    if ($Options->{ini}) {
        tie %Ini, 'Config::IniFiles', ( -file => $Options->{ini} )
            or die "$Options->{ini}: $!";
    }
    if ($Options->{ini} and not $Params->{SMTPServer}) {
        $Params->{SMTPServer} = $Ini{$Section}{"SMTPServer:$Host"};
        $Params->{SMTPServer} //= $Ini{$Section}{SMTPServer} // $Ini{LOOPS}{SMTPServer};
        if ($Params->{SMTPServer}) {
            $Params->{SMTPPort} = $Ini{$Section}{"SMTPPort:$Host"};
            $Params->{SMTPPort} //= $Ini{$Section}{SMTPPort} // $Ini{LOOPS}{SMTPPort};
            $Params->{SMTPLogin} = $Ini{$Section}{"SMTPLogin:$Host"};
            $Params->{SMTPLogin} //= $Ini{$Section}{SMTPLogin} // $Ini{LOOPS}{SMTPLogin};
            $Params->{SMTPPassword} = $Ini{$Section}{"SMTPPassword:$Host"};
            $Params->{SMTPPassword} //= $Ini{$Section}{SMTPPassword} // $Ini{LOOPS}{SMTPPassword};
        }
    }

    if ( $Options->{ini} and not $Params->{from} ) {
        $Params->{from} = $Ini{$Section}{"MailSender:$Host"};
        $Params->{from} //= $Ini{$Section}{MailSender} // $Ini{LOOPS}{MailSender};
    }
    my $Server = $Params->{SMTPServer} // 'localhost';
    my $Login = $Params->{SMTPLogin} // (getpwuid($<))[0];
    $Params->{from} //= $Login. '@'. $Server;

    if ( $Options->{ini} and not $Params->{to} ) {
        $Params->{to} = $Ini{$Section}{"MailRecipient:$Host"};
        $Params->{to} //= $Ini{$Section}{MailRecipient} // $Ini{LOOPS}{MailRecipient};
    }
    $Params->{to} //= $Params->{from};

    if ( $Options->{ini} and not $Params->{store} ) {
        $Params->{store} = $Ini{$Section}{"MailStore:$Host"};
        $Params->{store} //= $Ini{$Section}{MailStore} // $Ini{LOOPS}{MailStore};
    }

    return $Params;
}

1;
