use Test::More tests => 8;

use strict;
use File::Temp qw(tempfile);

use DE_EPAGES::Core::API::Error qw ( ExistsError );
use DE_EPAGES::Core::API::File qw ( WriteFile DeleteFile );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Core::API::Scheduler qw(GetMailParameters);
use DE_EPAGES::Database::API::Connection qw( ConnectSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

RunScript(
    Sub => \&Main
);

sub Main {
    # ------------------------------------------------------------------------
    # parameters from command line
    # (wrapScheduler.PL, SendProgramOutput.pl, SMTPSendProgramOutput.PL)
    # ------------------------------------------------------------------------

    my $CmdLineParams = { ##no critic qw(ProhibitUnusedVariablesEnhanced)
        ini          => 'path to Scheduler.conf',
        section      => 'section in Scheduler.conf',
        hostname     => 'hostname to use for key:hostname= in Scheduler.conf',
        store        => 'store to use to get from (ReplyAddress) and to (ErrorAddress)',
        from         => 'e-mail address',
        to           => 'list of addressees, separated by [,;]',
        cc           => 'list of addressees, separated by [,;]',
        bcc          => 'list of addressees, separated by [,;]',
        subject      => 'e-mail subject',
        content      => 'e-mail body',
        file         => 'content of file is e-mail body',
        descr        => 'additional description',
        attachments  => 'attachment files',
        SMTPServer   => 'send mail via server',
        SMTPPort     => 'port of SMTPServer',
        SMTPLogin    => 'login for SMTPServer',
        SMTPPassword => 'password for SMTPServer',
    };

    # ------------------------------------------------------------------------
    # parameters from Scheduler.conf (mapped to command line parameters)
    # command line parameters have higher precedence
    # ------------------------------------------------------------------------

    my $ConfigParams = { ##no critic qw(ProhibitUnusedVariablesEnhanced)
        MailStore     => 'store',
        MailSender    => 'from',
        MailRecipient => 'to',
        SMTPServer    => 'SMTPServer',
        SMTPPort      => 'SMTPPort',
        SMTPLogin     => 'SMTPLogin',
        SMTPPassword  => 'SMTPPassword',
    };

    # ------------------------------------------------------------------------
    # parameters used by DE_EPAGES::MailType::API::TextMail::SendTextMail
    # ------------------------------------------------------------------------

    ##no critic qw(RequireInterpolationOfMetachars)
    my $Localhost = '@localhost';
    my $MySelf = 'my@self';
    ##use critic

    my $MailParams = { ##no critic qw(ProhibitUnusedVariablesEnhanced)
        file         => 'if set, run _sendTBOErrorMail',
        descr        => 'dito',
        attachments  => 'dito',
        store        => 'dito, get "from" and "to" from store (default: Site) unless "to" defined',
        from         => 'e-mail address (default: to)',
        to           => 'unless set: if _sendTBOErrorMail then get ErrorAddress from store else whoami'. $Localhost,
        cc           => 'address list',
        bcc          => 'address list',
        subject      => 'e-mail subject',
        content      => 'e-mail body',
        SMTPServer   => 'unless set, get SMTPServer and other SMTP* in SmtpConnection from database',
        SMTPPort     => 'port of SMTPServer',
        SMTPLogin    => 'login for SMTPServer',
        SMTPPassword => 'password for SMTPServer',
    };

    # ------------------------------------------------------------------------
    # tests
    # ------------------------------------------------------------------------

    my ($Expected, $Result);
    $ENV{EPAGES_TESTSITE} //= 'Site';
    my $TestSite = $ENV{EPAGES_TESTSITE};

    $Result = GetMailParameters({});
    $Expected = {
        from => (getpwuid($<))[0]. $Localhost,
        to =>  (getpwuid($<))[0]. $Localhost,
    };
    is_deeply($Expected, $Result, 'GetMailParameters({})');

    $Result = GetMailParameters({ from => $MySelf });
    $Expected = {
        from => $MySelf,
        to => $MySelf,
    };
    is_deeply($Expected, $Result, "GetMailParameters({from => '$MySelf'})");

    $Result = GetMailParameters({ store => $TestSite });
    $Expected = GetMailParametersFromRootObject($TestSite);
    is( $Result->{store}, $TestSite, "GetMailParameters({store => '$TestSite'})" );
    is_deeply( [qw(SMTPLogin SMTPPassword SMTPPort SMTPServer from store to)],
               [sort keys %$Expected], "GetMailParametersFromRootObject('$TestSite')" );

    my (undef, $TmpFile) = tempfile();
    my $Content = "[LOOPS]\n";
    WriteFile($TmpFile, \$Content, {binary=>1, create_directory=>1});

    $Result = GetMailParameters({ ini => $TmpFile });
    $Expected = {
        from => (getpwuid($<))[0]. $Localhost,
        to => (getpwuid($<))[0]. $Localhost,
        store => undef,
        SMTPServer => undef
    };
    is_deeply($Expected, $Result, "GetMailParameters({ ini => '$TmpFile' })");

    $Content = <<'END_CONTENT';
[LOOPS]
MailStore=LoopsStore
MailSender=LoopsSender
MailRecipient=LoopsRecipient
SMTPServer=LoopsServer
SMTPPort=LoopsPort
SMTPLogin=LoopsLogin
SMTPPassword=LoopsPassword
[CronJob]
MailStore=CronStore
MailSender=CronSender
MailRecipient=CronRecipient
SMTPServer=CronServer
SMTPPort=CronPort
SMTPLogin=CronLogin
SMTPPassword=CronPassword
END_CONTENT

    WriteFile($TmpFile, \$Content, {binary=>1, create_directory=>1});
    $Result = GetMailParameters({ ini => $TmpFile });
    $Expected = {
        'from' => 'LoopsSender',
        'to' => 'LoopsRecipient',
        'store' => 'LoopsStore',
        'SMTPLogin' => 'LoopsLogin',
        'SMTPPassword' => 'LoopsPassword',
        'SMTPPort' => 'LoopsPort',
        'SMTPServer' => 'LoopsServer',
    };
    is_deeply($Expected, $Result, "GetMailParameters({ ini => '$TmpFile' })");

    $Result = GetMailParameters({ ini => $TmpFile, section => 'missing' });
    is_deeply($Expected, $Result, "GetMailParameters({ ini => '$TmpFile', section => 'missing' })");

    $Result = GetMailParameters({ ini => $TmpFile, section => 'CronJob' });
    $Expected = {
        'from' => 'CronSender',
        'to' => 'CronRecipient',
        'store' => 'CronStore',
        'SMTPLogin' => 'CronLogin',
        'SMTPPassword' => 'CronPassword',
        'SMTPPort' => 'CronPort',
        'SMTPServer' => 'CronServer',
    };
    is_deeply($Expected, $Result, "GetMailParameters({ ini => '$TmpFile', section => 'CronJob' })");

    DeleteFile($TmpFile);
}

sub GetMailParametersFromRootObject {
    my ($TestSite) = @_;
    my $hParameters;

    eval { ConnectSite($TestSite); };
    if (ExistsError()) {
        $hParameters = {
            'from' => 'root',
            'to' => 'root',
            'SMTPServer' => 'localhost',
            'SMTPPort' => 25,
            'SMTPLogin' => 'root',
            'SMTPPassword' => undef,
        };
    } else {
        my $System = LoadRootObject();
        my %Map = (ReplyAddress => 'from', ErrorAddress => 'to');
        $hParameters->{$Map{$_} // $_} = $System->get($_)
            foreach qw(ReplyAddress ErrorAddress SMTPServer SMTPPort SMTPLogin SMTPPassword);
        DisconnectSite();
    }
    $hParameters->{store} = $TestSite;
    return $hParameters;
}

1;
