#========================================================================================
# package      DE_EPAGES::MailType::API::Object::SmtpConnection
# state        public
# base         DE_EPAGES::Mail::API::SMTP
#----------------------------------------------------------------------------------------
# description  object interface for an SMTP Connection.
#----------------------------------------------------------------------------------------
# example      my $Smtp = DE_EPAGES::MailType::API::Object::SmtpConnection->new;
#               $Smtp->connection( sub {
#                   $Smtp->send( "This is a test", {
#                       'FROM'      => 'Sender <mailer@epages.de>',
#                       'TO'        => ['Recipient <rcpt@epages.de>'],
#                       'SUBJECT'   => 'Hallo',
#                   } );
#               });
#========================================================================================
package DE_EPAGES::MailType::API::Object::SmtpConnection;
use base qw( DE_EPAGES::Mail::API::SMTP );

use strict;
use DE_EPAGES::Core::API::TestParameter qw ( TestHash );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject);
#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Connection = DE_EPAGES::MailType::API::Object::SmtpConnection->new();
# syntax       $Connection = DE_EPAGES::MailType::API::Object::SmtpConnection->new( $hParameters );
#----------------------------------------------------------------------------------------
# example      my $Connection = DE_EPAGES::MailType::API::Object::SmtpConnection->new({
#                   'SMTPServer'   => 'mailgate.epages.de',
#                   'SMTPPort'     => 25,
#                   'SMTPLogin'    => 'mmustermann',
#                   'SMTPPassword' => '123456',
#               });
#----------------------------------------------------------------------------------------
# description  Creates a new SmtpConnection object using the SMTP server that is
#               configured by the attributes SMTPServer, SMTPPort, SMTPLogin and
#               SMTPPassword of the System object unless these parameters are passed to
#               the sub-routine.<br />
#----------------------------------------------------------------------------------------
# input        $hParameters | custom SMTP attributes (optional), with following keys:
#               <ul>
#                   <li>SMTPServer | SMTP server name | string
#                   <li>SMTPPort | SMTP port | integer
#                   <li>SMTPLogin | (optional) SMTP login | string
#                   <li>SMTPPassword | (optional) SMTP password | string
#               </ul> | reference to hash
# return       $Connection | SMTP connection object | object
#========================================================================================
sub new {
    my $class = shift;
    my ($hParameters) = @_;

    unless (defined $hParameters->{SMTPServer}) {
         my $System = LoadRootObject();
         $hParameters = $System->get([ qw(SMTPServer SMTPPort SMTPLogin SMTPPassword) ]);
    }
    TestHash( 'hParameters', $hParameters, {
        'SMTPServer'   => 'String;NotNull',
        'SMTPPort'     => 'Integer;NotNull',
        'SMTPLogin'    => 'String',
        'SMTPPassword' => 'String',
    });

    return $class->SUPER::new(
        $hParameters->{'SMTPServer'},
        $hParameters->{'SMTPPort'},
        $hParameters->{'SMTPLogin'},
        $hParameters->{'SMTPPassword'}
    );
}

#========================================================================================
# function     send
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Connection->send( $MailBody, $hParams );
# example      $Connection->send( $Mail->createMail(), $Mail->getHeader());
#----------------------------------------------------------------------------------------
# description  Send the e-mail using the SMPT connection
#----------------------------------------------------------------------------------------
# input        $MailBody | MIME message body | string
# input        $hParams | E-Mail headers and envelope parameters, see
#                       <function DE_EPAGES::Mail::API::SMTP::sendDataToSMTP>
#                        | ref.hash.string
#========================================================================================
sub send {
    my $self = shift;
    my ($MailBody, $hParams) = @_;

    $self->SUPER::sendDataToSMTP($MailBody,$hParams);
    return;
}

#========================================================================================
# function     open
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Connection->open();
#----------------------------------------------------------------------------------------
# description  obsolete. Use connection() instead.
#========================================================================================
sub open {
    my $self = shift;

    $self->SUPER::openConnection();
    return;
}

#========================================================================================
# function     close
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Connection->close();
#----------------------------------------------------------------------------------------
# description  obsolete. Use connection() instead.
#========================================================================================
sub close { ## no critic qw(ProhibitAmbiguousNames)
    my $self = shift;

    $self->SUPER::closeConnection();
    return;
}

1;
