#========================================================================================
# package      DE_EPAGES::MailType::API::TextMail
# state        public
#----------------------------------------------------------------------------------------
# description  Sends non-html e-mails containing any number of attachments.
#========================================================================================
package DE_EPAGES::MailType::API::TextMail;
use base Exporter;
use strict;

our @EXPORT_OK = qw(
    SendTextMail
);

use DE_EPAGES::Mail::API::Mail;
use DE_EPAGES::MailType::API::Object::SmtpConnection;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::File qw ( GetFileContent ExistsFile );
use DE_EPAGES::Core::API::String qw (SplitEMailAddresses);
use DE_EPAGES::Database::API::Connection qw( ConnectSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::Warning qw( Warning );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'MISSING_ADDRESS' => 'Can\'t send e-mail: No error e-mail set in TBO',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     SendTextMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       SendTextMail(\%Options)
#----------------------------------------------------------------------------------------
# description  This function e-mails a file.
#----------------------------------------------------------------------------------------
# input        $Option | options keys:
#               <ul>
#                 <li>from     - sender (e-mail address) - string
#                 <li>to       - list of addressees, separated by [,;] - String
#                 <li>cc       - list of addressees, separated by [,;] - String
#                 <li>bcc      - list of addressees, separated by [,;] - String
#                 <li>subject  - e-mail subject - String
#                 <li>content  - e-mail body - String
#                 <li>attachments - attachment files
#                   <ul>
#                     <li>filename  - path to file or file name - string</li>
#                     <li>mime_type - MIME type (default: Application/Octet-stream) - string</li>
#                     <li>data      - document to attach (optional) - binary</li>
#                     <li>encoding  - document's encoding (optional, default: base64) - string</li>
#                   </ul>
#                 </li>
#                 <li>SMTPServer   - optional, default from database</li>
#                 <li>SMTPPort     - optional, default from database</li>
#                 <li>SMTPLogin    - optional, default from database</li>
#                 <li>SMTPPassword - optional, default from database</li>
#               </ul> | ref.array.hash
#========================================================================================
sub SendTextMail {
    my ($hOptions) = @_;

    # compatibility mode for obsolete parameters
    if($hOptions->{file} || $hOptions->{descr} ||
       $hOptions->{attachments} || $hOptions->{store}){
        _sendTBOErrorMail($hOptions);
        return;
    }

    my $From = $hOptions->{'from'};
    unless(defined $From){
        Warning('MISSING_FROM');
        return;
    }

    my $To = $hOptions->{'to'};
    unless(defined $To){
        Warning('No_Recipients');
        return;
    }

    my $aTo = SplitEMailAddresses($To);
    my $aCc = defined $hOptions->{'cc'} ? SplitEMailAddresses($hOptions->{'cc'}) : undef;
    my $aBcc = defined $hOptions->{'bcc'} ? SplitEMailAddresses($hOptions->{'bcc'}) : undef;

    my $Mail = DE_EPAGES::Mail::API::Mail->new({
          'FROM'    => $From,
          'TO'      => $aTo,
          'CC'      => $aCc,
          'BCC'     => $aBcc,
          'SUBJECT' => $hOptions->{'subject'},
      });

    $Mail->addText({'content' => $hOptions->{'content'} || ''});

    if(defined $hOptions->{'attachments'}){
        foreach my $hAttachment (@{$hOptions->{'attachments'}}){
            if(ExistsFile($hAttachment->{'filename'})){
                $Mail->addAttachment([{
                    'filename'  => $hAttachment->{'filename'},
                    'type'      => $hAttachment->{'mime_type'},
                }]);
            } elsif(defined $hAttachment->{'data'}){
                $Mail->addAttachment([{
                    'filename'  => $hAttachment->{'filename'},
                    'type'      => $hAttachment->{'mime_type'},
                    'data'      => $hAttachment->{'data'},
                    'encoding'  => $hAttachment->{'encoding'},
                }]);
            }
        }
    }

    my $Connection = DE_EPAGES::MailType::API::Object::SmtpConnection->new($hOptions);
    $Connection->connection( sub {
        $Connection->send( $Mail->createMail(), $Mail->getHeader());
    });

    return;
}

#========================================================================================
# DEPRECATED
#----------------------------------------------------------------------------------------
# function     _sendTBOErrorMail
#----------------------------------------------------------------------------------------
# syntax       _sendTBOErrorMail(\%Options)
#----------------------------------------------------------------------------------------
# description  This function e-mails a file.
#----------------------------------------------------------------------------------------
# input        $Option | options keys:
#               <ul>
#                 <li>store    - use the e-mail connection as defined in store (default: Site)
#                 <li>from     - sender (e-mail address) (default: TBO Reply Address)
#                 <li>to       - list of addressees, separated by [,;] (default: TBO Error Address)
#                 <li>subject  - e-mail subject
#                 <li>file     - file to send
#                 <li>descr    - additional description
#                 <li>attachment - file is sent as attachment (default: Application/Octet-stream)
#               </ul> | hash
#========================================================================================
sub _sendTBOErrorMail {
    my ($Options) = @_;

    $Options->{subject} //= 'Notification from ePages';

    unless ($Options->{to}) {
        my $Store = $Options->{store} ? $Options->{store} : 'Site';
        ConnectSite($Store);
        my $System = LoadRootObject();

        $Options->{to} = $System->get('ErrorAddress');
        $Options->{from} //= $System->get('ReplyAddress');
        DisconnectSite();
    }
    $Options->{from} //= $Options->{to};

    Error('MISSING_ADDRESS') unless $Options->{from};

    my $Mail = DE_EPAGES::Mail::API::Mail->new({
          'SUBJECT'    => $Options->{subject},
          'FROM'       => $Options->{from},
          'TO'         => SplitEMailAddresses($Options->{to}),
      });

    $Options->{descr} .= "\n\n" if $Options->{descr};
    if (defined $Options->{attachment}) {
        $Options->{descr} .= "(attached: $Options->{file})\n";
    } else {
        my $FileContent = GetFileContent($Options->{file});
        $Options->{descr} .= $$FileContent;
    }

    $Mail->addText({
        'content'     => $Options->{descr},  # mandatory
        'encoding'    => 'quoted-printable', # optional (default: quoted-printable)
        'type'        => 'text/plain',       # optional (default: text/plain)
        'type.charset'=> 'utf-8',            # optional (default: utf-8)
    });

    if (defined $Options->{attachment}) {
        $Mail->addAttachment([{
            'filename'  => $Options->{file},
            'type'      => $Options->{attachment},
        }]);
    }

    my $Connection = DE_EPAGES::MailType::API::Object::SmtpConnection->new($Options);
    $Connection->connection( sub {
        $Connection->send( $Mail->createMail(), $Mail->getHeader());
    });

}

1;
