#==============================================================================
# package      DE_EPAGES::Payment::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::Payment::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject Transaction );
use DE_EPAGES::Payment::API::Constants qw ( $NUMBER_OF_RATES_LIST );
use DE_EPAGES::Payment::API::Shop qw ( GetPaymentMethodsFolder );
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw (
    DeleteShippingPaymentMap
    ExistsShippingPaymentMap
    InsertShippingPaymentMap
);
use DE_EPAGES::Shop::API::ShopUtil;

#==============================================================================
# Event handlers
#==============================================================================

sub _uniquePaymentMethodAlias {
    my $self = shift;
    my ($Shop, $PaymentType) = @_;

    my $Folder = $Shop->child('PaymentMethods');

    my $PaymentTypeName = $PaymentType->get('NameOrAlias');
    return $PaymentTypeName unless $Folder->existsChild( $PaymentTypeName );

    my $Counter = 1;
    while( 1 ) {
        my $Alias = $PaymentTypeName.'_'.$Counter;
        return $Alias unless $Folder->existsChild( $Alias );
        $Counter++;
    }
}

sub NewPaymentMethod {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    SavePaymentMethods($self,$Servlet);

    my $hForm = $Servlet->form->form($Shop, 'NewPaymentMethod');
    my $PaymentType = LoadRootObject()->loadObject($hForm->{'NewPaymentTypeID'}, 'PaymentType');
    my $Folder = $Shop->child('PaymentMethods');
    my $NewPosition = (defined $hForm->{'NewPosition'}) ? $hForm->{'NewPosition'} : 9999;
    my %Values = (
        # for the alias use the alias of the payment type plus counter if alias already exists
        'Alias' => $self->_uniquePaymentMethodAlias( $Shop, $PaymentType ),
        'Name' => $hForm->{'NewAlias'},
        'Position' => $NewPosition,
        'IsDefault' => $Folder->hasChildren ? 0 : 1,
    );

    my $PaymentMethodId = undef;

    eval {
        Transaction( sub {
            my $PaymentMethod = $PaymentType->newPaymentMethod($Shop, \%Values);
            $PaymentMethodId = $PaymentMethod->id();
            return unless defined $PaymentMethod;

            # set name for all languages in a Unity Shop
            if ($Shop->get('IsUnityRegistered')) {
                my $Name = $hForm->{'NewAlias'};
                if (defined $Name) {
                    my @ShopLanguageIDs = map { $_->{'LanguageID'} } @{$Shop->get('Languages')};
                    if (scalar @ShopLanguageIDs > 1) {
                        my $FormLanguageID = $Servlet->form->value('LanguageID');
                        foreach my $LanguageID (@ShopLanguageIDs) {
                            next if $LanguageID == $FormLanguageID;
                            $PaymentMethod->set({'Name' => $Name}, $LanguageID);
                        }
                    }
                }
            }

            # activate on all shipping methods
            InsertShippingPaymentMap({'ShippingMethodID'=> $_->id, 'PaymentMethodID'=> $PaymentMethod->id}) foreach @{$Shop->get('ShippingMethods')};

        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
        } elsif ($Error->code eq 'NoSecondInstanceAllowed') {
            $Servlet->vars('NoSecondInstanceAllowed',1);
        } else {
            Error($Error);
        }
    }

    $Shop->child('PaymentMethods')->renumberChildren;

    if($PaymentMethodId) {
      my $ViewAction = $Servlet->params('RedirectToPaymentMethodViewAction');
      if(isValid($ViewAction)){
          $Servlet->vars('ViewAction', $ViewAction);
          $Servlet->vars('ObjectID', $PaymentMethodId);
      }
    }

    return;
}

sub setDefault {
    my $self = shift;
    my ($PaymentMethod) = @_;

    # all Payment methods
    my $ShopMethods = $PaymentMethod->getSite()->get('PaymentMethods');

    # if only one then this is default
    if (scalar @$ShopMethods == 1) {
        $PaymentMethod->set({'IsDefault'=>1});
        return;
    }

    # all defaults without current
    my @DefaultMethods = grep { $_->get('IsDefault') and $PaymentMethod->id != $_->id } @$ShopMethods;

    # current default
    my $IsDefault = $PaymentMethod->get('IsDefault');

    if ($IsDefault) {
        # set all non current to non default
        $_->set({'IsDefault'=>0}) foreach @DefaultMethods;
    } elsif (scalar @DefaultMethods > 1) {
        # multible defaults to non default
        shift @DefaultMethods;
        $_->set({'IsDefault'=>0}) foreach @DefaultMethods;
    } else { # scalar @DefaultMethods == 0
        # set first not current to default
        my @NewDefaultMethods = grep { $PaymentMethod->id != $_->id } @$ShopMethods;
        $NewDefaultMethods[0]->set({'IsDefault'=>1});
    }
    return;
}
sub ViewShippingPaymentMap {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $PaymentMethods  = $Shop->get('PaymentMethods');
    my $ShippingMethods = $Shop->get('ShippingMethods');
    my @PaymentLoop;
    my $PaymentWithoutShippingExist = 0;
    foreach my $PaymentMethod (@$PaymentMethods) {
        $PaymentWithoutShippingExist = 1 if ($PaymentMethod->get('WorksWithoutShipping'));
        my $PaymentHash = $PaymentMethod->tleHash;
        push @PaymentLoop, $PaymentHash;

        my @ShippingLoop;
        $PaymentHash->{'ShippingMethods'} = \@ShippingLoop;
        foreach my $ShippingMethod (@$ShippingMethods) {
            my $ShippingHash = $ShippingMethod->tleHash;
            push @ShippingLoop, $ShippingHash;

            $ShippingHash->{'IsEnabled'} = ExistsShippingPaymentMap($ShippingMethod->id, $PaymentMethod->id);
            $ShippingHash->{'InputName'} = join('_','IsEnabled', $ShippingMethod->id, $PaymentMethod->id);
        }
    }
    $Servlet->vars('PaymentWithoutShippingExist', $PaymentWithoutShippingExist);
    $Servlet->vars('PaymentMethods', \@PaymentLoop);
    return;
}

sub UpdateShippingPaymentMap {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    my $PaymentMethods  = $Shop->get('PaymentMethods');
    my $ShippingMethods = $Shop->get('ShippingMethods');
    my @WorksWithoutShipping = $Servlet->form->value('WorksWithoutShipping');

    my $CreateError = 1;
    foreach my $PaymentMethod (@$PaymentMethods) {
        $CreateError = 0 if ElementOf($PaymentMethod->id, @WorksWithoutShipping);
        foreach my $ShippingMethod (@$ShippingMethods) {
            my $InputName = join('_','IsEnabled', $ShippingMethod->id, $PaymentMethod->id);
            $CreateError = 0 if $Servlet->params($InputName);
        }
    }
    if ($CreateError) {
        $Servlet->vars('NoDependencies', 1);
        return;
    }
    foreach my $PaymentMethod (@$PaymentMethods) {
        $PaymentMethod->set({'WorksWithoutShipping'=> ElementOf($PaymentMethod->id, @WorksWithoutShipping) ? 1 : 0});
        foreach my $ShippingMethod (@$ShippingMethods) {
            my $InputName = join('_','IsEnabled', $ShippingMethod->id, $PaymentMethod->id);
            my $ExistsMap = ExistsShippingPaymentMap($ShippingMethod->id, $PaymentMethod->id);
            my $ExistsForm = $Servlet->params($InputName);
            if ($ExistsForm xor $ExistsMap) {
                my %Info = ('ShippingMethodID' => $ShippingMethod->id, 'PaymentMethodID' => $PaymentMethod->id);
                if ($ExistsForm) {
                    InsertShippingPaymentMap(\%Info);
                } else {
                    DeleteShippingPaymentMap(\%Info);
                }
            } # else do nothing
        }
    }
}

sub SavePaymentMethods {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $DefaultMethodID = $Servlet->params('IsDefault');
    my $ahListItems = $self->listItems( $Servlet, $Shop, 'PaymentMethod' );

    # test if default is visible
    if (defined $DefaultMethodID) {
        my $IsDefaultVisible = $Shop->loadObject($DefaultMethodID, 'PaymentMethod')->get('IsActivated');
        return $Servlet->vars('INVISIBLE_DEFAULT',1) unless $IsDefaultVisible;
    }

    # save default and position
    foreach my $hListItem (@$ahListItems) {
        my $Method = $hListItem->{'Object'};
        $Method->set({
            'IsDefault' => $DefaultMethodID == $Method->id ? 1 : 0,
            'Position' => $hListItem->{'Position'},
        });
    }
    GetPaymentMethodsFolder($Shop)->renumberChildren;

    return;
}

#------------------------------------------------------------------------------------------------------
sub ViewHirePurchase {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    my $CurrencyID = $Servlet->form->value('CurrencyID');
    $CurrencyID = $Servlet->form->value('Alias') unless defined $CurrencyID;;
    $CurrencyID = $Shop->get('CurrencyID') unless defined $CurrencyID;

    $Servlet->vars('Rates', [map { {'NumberOfMonths' => $_} } (@$NUMBER_OF_RATES_LIST)] );
    $Servlet->vars('SelectedCurrencyID', $CurrencyID);

    my $Folder = $Shop->folder('InterestRateMatrices');
    my $InterestRateMatrix = $Folder->existsChild($CurrencyID)  ?  $Folder->child($CurrencyID) : undef; # currencyid is alias

    my (@aRatesLoop) = ();
    if (defined $InterestRateMatrix) {
        my $aRates = $InterestRateMatrix->get('InterestRates');

        my %hRates;
        $hRates{$_->get('MinimumAmount')*1}{$_->get('NumberOfMonths')*1} = $_ foreach @$aRates;
        foreach my $amount(sort {$a <=> $b } keys %hRates) {
            my @aRateValues = ();
            foreach my $months(@$NUMBER_OF_RATES_LIST) {

                my $hInfo = defined $hRates{$amount}{$months}
                    ? $hRates{$amount}{$months}->tleHash
                    : {
                        'InterestRateValue' => undef,
                        'MinimumAmount'     => $amount,
                        'NumberOfMonths'    => $months
                    };

                $hInfo->{'InterestRateValue'} *= 100 if defined $hInfo->{'InterestRateValue'}; ## no critic qw(:ProhibitMagicNumbers)
                push(@aRateValues, {'InterestRate' => $hInfo});
            }
            push @aRatesLoop, {'MinimumAmount' => $amount,
                               'InterestRates' => \@aRateValues};
        }
        $Servlet->vars('InterestRateMatrix', $InterestRateMatrix->tleHash);
    }
    $Servlet->vars('InterestRatesRows', \@aRatesLoop);
}

#------------------------------------------------------------------------------------------------------
sub SaveHirePurchase {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $FormValues = $Servlet->form->form($Shop, 'HirePurchase');
    my $CurrencyID = $FormValues->{'Alias'};
    my $Folder = $Shop->folder('InterestRateMatrices');

    my $isNewMinimumAmount = $Servlet->params('NewMinimumAmount') =~ /\S/ ? 1:0;
    # for new minimum amount check if we already have a line with this minimum amount
    if ($isNewMinimumAmount) {
        my $InterestRateMatrix = $Folder->existsChild($CurrencyID)  ?  $Folder->child($CurrencyID) : undef;
        if (defined($InterestRateMatrix)) {
            my $Site = $InterestRateMatrix->getSite();
            my $LocaleID = $Site->get('LocaleID');
            my $Locale   = $Site->siteLocale($LocaleID);
            my $FormValues2  = $Servlet->form->form($Shop, 'NewAmountRange');
            my $minAmount = $Locale->roundMoney($FormValues2->{'NewMinimumAmount'}, $CurrencyID);
            if (scalar(grep { $_->get('MinimumAmount') == $minAmount } @{$InterestRateMatrix->get('InterestRates')}) > 0) {
                $Servlet->form->executeFormError({
                    'Form'   => 'HirePurchase',
                    'Reason' => 'MINAMOUNT_EXISTS',
                    'Name'   => 'NewMinimumAmount',
                    'Value'  => $minAmount
                }) ;
            }
        }
        my $count = 0;
        my @InterestRateValue = $Servlet->form->value('NewInterestRateValue');

        foreach my $InterestRateValue (@InterestRateValue){
            if ($InterestRateValue ne '') {
                $count = $count + 1;
            }
        }
        if ( ($Servlet->form->value('NewMinimumAmount') > 0) and ($count == 0)){
            $Servlet->form->executeFormError({
                'Form'   => 'HirePurchase',
                'Reason' => 'NO_RATE_VALUE',
                'Name'   => 'NewInterestRateValue',
            }) ;
        }
    }


    # localized InfoText
    my $hFormValuesLanguage = $Servlet->form->form($Shop, 'PaymentInfoText');
    foreach my $hLanguage (@{$hFormValuesLanguage->{'Languages'}}) {
        $Shop->set({'DeferredPaymentInfoText' => $hLanguage->{'DeferredPaymentInfoText'}}, $hLanguage->{'LanguageID'});
    }

    # existing amount ranges
    if (defined($Servlet->params('MinimumAmount')) and $Servlet->params('MinimumAmount') =~ /[^\0]+/ and $Servlet->params('MinimumAmount') !~ /^\s*$/) {
        my $InterestRateMatrix = $Folder->child($CurrencyID);
        setInterestRateValues($InterestRateMatrix, $FormValues->{'InterestRates'});
    }

    # new amount range
    if ($isNewMinimumAmount) {
        NewHirePurchase($self, $Servlet);
    }
    return;
}

#------------------------------------------------------------------------------------------------------
sub NewHirePurchase {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $FormValues  = $Servlet->form->form($Shop, 'NewAmountRange');
    my $MinimumAmount = $FormValues->{'NewMinimumAmount'};
    my $CurrencyID    = $FormValues->{'Alias'};

    my $Folder = $Shop->folder('InterestRateMatrices');
    my $InterestRateMatrix = $Folder->existsChild($CurrencyID)  ?  $Folder->child($CurrencyID) : undef; # currencyid is alias
    #my ($InterestRateMatrix) = grep {$_->{'CurrencyID'} eq $CurrencyID} @{$Shop->get('InterestRateMatrices')};

    unless (defined $InterestRateMatrix) {
        my $Class   = LoadClassByAlias('InterestRateMatrix');
        $InterestRateMatrix = $self->SUPER::NewObject($Servlet, $Folder, $Class);
    }

    my @aNewEntries = ();
    foreach my $Entry (@{$FormValues->{'NewInterestRates'}}) {
        push(@aNewEntries, {'InterestRateValue' => $Entry->{'NewInterestRateValue'},
                            'NumberOfMonths'    => $Entry->{'NewNumberOfMonths'},
                            'MinimumAmount'     => $MinimumAmount});
    }

    setInterestRateValues($InterestRateMatrix, \@aNewEntries);

    return;
}

#------------------------------------------------------------------------------------------------------
sub setInterestRateValues {
    my ($InterestRateMatrix, $aEntries) = @_;
    LogDebug('aEntries', $aEntries);

    my $aInterestRates = $InterestRateMatrix->get('InterestRates');
    my $Site = $InterestRateMatrix->getSite();
    my $LocaleID = $Site->get('LocaleID');
    my $Locale   = $Site->siteLocale($LocaleID);
    my $CurrencyID = $InterestRateMatrix->alias;

    my %hRates;
    $hRates{$_->get('MinimumAmount')*1}{$_->get('NumberOfMonths')*1} = $_ foreach @$aInterestRates;
    my $InterestRateClass = LoadClassByAlias('InterestRate');
    foreach my $Entry (@$aEntries) {
        my $InterestRateValue  = $Entry->{'InterestRateValue'};
        my $NoOfMonths         = $Entry->{'NumberOfMonths'} * 1;
        my $MinimumAmount      = $Locale->roundMoney($Entry->{'MinimumAmount'}*1, $CurrencyID);

        if (defined $hRates{$MinimumAmount}{$NoOfMonths}) {
            if (defined $InterestRateValue) {
                #--- update
                $hRates{$MinimumAmount}{$NoOfMonths}->set({'InterestRateValue' => $InterestRateValue / 100}); ## no critic qw(:ProhibitMagicNumbers)
            }
            else {
                #--- delete
                $hRates{$MinimumAmount}{$NoOfMonths}->delete();
                delete($hRates{$MinimumAmount}{$NoOfMonths});
            }
        }
        else {
            #--- insert if defined
            $InterestRateClass->insertObject ({
                'InterestRateMatrix' => $InterestRateMatrix,
                'MinimumAmount'      => $MinimumAmount,
                'NumberOfMonths'     => $NoOfMonths,
                'InterestRateValue'  => $InterestRateValue / 100  ## no critic qw(:ProhibitMagicNumbers)
            }) if defined $InterestRateValue;
        }
    }

    return;
}

sub SetPaymentMethodVisible {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my @ListObjectIDs = $Servlet->form->value('ListObjectID');

    my $aShopCurrencies  = $Shop->get('Currencies');
    my @ActiveCurrencies = grep { $_->{'IsActivated'} == 1 } @$aShopCurrencies;
    my $hVars = { 'TaxModel' => $Shop->get('TaxModel') };

    foreach my $PaymentMethod (@{$Shop->loadObjects( \@ListObjectIDs, 'PaymentMethod' )}) {
        foreach my $ActiveCurrency (@ActiveCurrencies) {
            $hVars->{'CurrencyID'} = $ActiveCurrency->{'CurrencyID'};
            my $SettingsComplete = $PaymentMethod->usableAtStorefront($hVars);
            if ($SettingsComplete == 0) {
                $Servlet->form->executeFormError({
                    'Reason'=> 'SETTINGS_INCOMPLETE',
                    'Name'  => 'Alias',
                    'Value' => $PaymentMethod->alias,
                });
            }
        }
        $PaymentMethod->set({ 'IsActivated' => 1 });
    }
    return;
}

sub SetPaymentMethodNotVisible {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my @ListObjectIDs = $Servlet->form->value('ListObjectID');

    foreach my $PaymentMethod (@{$Shop->loadObjects(\@ListObjectIDs, 'PaymentMethod')}) {
        if (defined $Servlet->params('IsDefault')) {
            my $DefaultPaymentMethod = $Shop->loadObject($Servlet->params('IsDefault'), 'PaymentMethod');
            $DefaultPaymentMethod->id == $PaymentMethod->id
                ? $Servlet->vars('INVISIBLE_DEFAULT',1)
                : $PaymentMethod->set({ 'IsActivated' => 0 });
        } else {
            $Servlet->vars('INVISIBLE_DEFAULT',1);
        }
    }

    return;
}

sub DeletePaymentMethods {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    $self->SUPER::DeleteListedObjects($Servlet);

    $Shop->child('PaymentMethods')->renumberChildren;

    return;
}
#========================================================================================
# function     UnityMBOViewPaymentList
# state        public
#----------------------------------------------------------------------------------------
# description  ViewAction for payment selection page
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#========================================================================================
sub UnityMBOViewPaymentList {
    my $self = shift;
    my ($Servlet) = @_;

        my $Shop = $Servlet->object();
        my $ShopUtil = $self->_createShopUtil($Shop);
        $Servlet->vars('MaxFeatureValuePaymentMethodsReached', $ShopUtil->maxFeatureValuePaymentMethodsReached($Shop));

    return;
}
#========================================================================================
# function     ViewPaymentGroupDetails
# state        public
#----------------------------------------------------------------------------------------
# description  ViewAction for payment group details page
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#========================================================================================
sub ViewPaymentGroupDetails {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $PaymentGroupID = $Servlet->params('PaymentGroupID');
    my $PaymentGroup = $Shop->loadObject($PaymentGroupID, 'PaymentGroup', {'AllowSystemSite' => 1});

    my $aPaymentTypes = $PaymentGroup->getPaymentTypes($Shop);
    my $hPaymentGroup = $PaymentGroup->tleHash();
    my $aPaymentTypesTleHash = [];

    foreach my $PaymentType (@{$aPaymentTypes}){
        my $hPaymentType = $PaymentType->tleHash();
        $hPaymentType->{'CanCreateAdditionalPaymentMethod'} = $PaymentType->canCreateAdditionalPaymentMethod($Shop);

        push(@{$aPaymentTypesTleHash}, $hPaymentType);
    }

    $hPaymentGroup->{'PaymentTypes'} = $aPaymentTypesTleHash;
    $Servlet->vars('PaymentGroup', $hPaymentGroup);
    my $ShopUtil = $self->_createShopUtil($Shop);
    $Servlet->vars('MaxFeatureValuePaymentMethodsReached', $ShopUtil->maxFeatureValuePaymentMethodsReached($Shop));

    return;
}
#----------------------------------------------------------------------------------------
sub _createShopUtil() {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::ShopUtil->new($Shop);
}

1;
