#========================================================================================
# package      DE_EPAGES::PaymentTypes::Hooks::Shop
# state        public
# description  handles the external payment shop notification
#========================================================================================
package DE_EPAGES::PaymentTypes::Hooks::Shop;
use base qw( Exporter );

use strict;
use DateTime::Duration;
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::PaymentTypes::API::Cartridge;

#========================================================================================
# function     OnBeforeCreateUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforeCreateUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_BeforeCreateUnityShop.
#               Imports the shop notification for the setup of an external payment
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Shop - shop object | object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeCreateUnityShop {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Object'};
    # import notifications
    my $Cartridge = DE_EPAGES::PaymentTypes::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/ShopNotifications.xml', $Shop->pathString);

    return;
}

#========================================================================================
# function     OnMBOViewMBO
#----------------------------------------------------------------------------------------
# syntax       OnMBOViewMBO( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook UI_MBOViewMBO.
#               Check for Unity shops if the notification for external payments expires
#               within 20 days after ahop creation and disables it in this case
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Shop - shop object | object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnMBOViewMBO {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Shop'};

    if ($Shop->get('IsUnityRegistered')) {

        return unless $Shop->folder('ShopNotifications')->existsChild('SetUpExternalPayment');

        my $SetUpPaymentMethodNotification
            = $Shop->folder('ShopNotifications')->child('SetUpExternalPayment');

        # only check notifications that have not been activated once
        # (indicated by an assigned object)
        unless ($SetUpPaymentMethodNotification->get('ShopNotificationObjectIDString')) {
            # activate notification 20 days after shop creation
            my $ShopCreationDate = $Shop->get('CreationDate');
            my $ShopCreationNormDay
                = $ShopCreationDate->clone->truncate( to => 'day' );
            my $Duration = DateTime::Duration->new( days  => 20 );
            my $NotificationReleaseDate = $ShopCreationNormDay + $Duration;
            my $Now = GetCurrentDBHandle()->currentDateTime;
            my $NowNormDay = $Now->clone->truncate( to => 'day' );

            if ($NowNormDay >= $NotificationReleaseDate) {
                # add shop object to notification to indicate
                # that it has been activated once
                $SetUpPaymentMethodNotification->addObject($Shop);
                # initialize notification according to the existing payment methods
                my $aPaymentMethods = $Shop->get('PaymentMethods');
                foreach my $PaymentMethod (@$aPaymentMethods) {
                    my $hInfoPaymentMethod
                        = $PaymentMethod->get(['UsableAtStorefront', 'PaymentType']);
                    # deactivate notification if a payment that is not standard
                    # has been completely set up
                    if ($hInfoPaymentMethod->{'PaymentType'}->get('PaymentGroup')->alias ne 'Standard'
                     && $hInfoPaymentMethod->{'UsableAtStorefront'}) {
                        $SetUpPaymentMethodNotification->deactivate();
                        last;
                    }
                }

            }
        }
    }

    return;
}

1;
