use strict;
use DateTime::Duration;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::PaymentTypes::API::Cartridge;
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         InitializeSetUpExternalPaymentNotification
# state            public
#----------------------------------------------------------------------------------------
# syntax           InitializeSetUpExternalPaymentNotification();
#----------------------------------------------------------------------------------------
# description      Adds the set up external payment notification for all Unity shops
#========================================================================================
sub InitializeSetUpExternalPaymentNotification {

    print "Reading all Unity shops\n";
    my $aShopIDs = GetUnityShopIDs();

    my $Cartridge = DE_EPAGES::PaymentTypes::API::Cartridge->new;
    foreach my $ShopID (@$aShopIDs) {
        eval {
            my $Shop = LoadObject($ShopID);
            # import notifications
            ImportObjects($Cartridge->xmlDirectory().'/ShopNotifications.xml', $Shop->pathString);
            # activate notification 20 days after shop creation
            my $ShopCreationDate = $Shop->get('CreationDate');
            my $ShopCreationNormDay = $ShopCreationDate->clone->truncate( to => 'day' );
            my $Duration = DateTime::Duration->new( days  => 20 );
            my $NotificationReleaseDate = $ShopCreationNormDay + $Duration;
            my $Now = GetCurrentDBHandle()->currentDateTime;
            my $NowNormDay = $Now->clone->truncate( to => 'day' );

            if ($NowNormDay >= $NotificationReleaseDate) {
                # add shop object to notification to indicate that it has been activated once
                my $SetUpPaymentMethodNotification
                    = $Shop->folder('ShopNotifications')->child('SetUpExternalPayment');
                $SetUpPaymentMethodNotification->addObject($Shop);
                # initialize notification according to the existing payment methods
                my $aPaymentMethods = $Shop->get('PaymentMethods');
                foreach my $PaymentMethod (@$aPaymentMethods) {
                    my $hInfoPaymentMethod
                        = $PaymentMethod->get(['UsableAtStorefront', 'PaymentType']);
                    # deactivate notification if a payment that is not standard
                    # has been completely set up
                    my $PaymentGroup = $hInfoPaymentMethod->{'PaymentType'}->get('PaymentGroup');
                    next unless defined $PaymentGroup;
                    if ($PaymentGroup->alias ne 'Standard' && $hInfoPaymentMethod->{'UsableAtStorefront'}) {
                        $SetUpPaymentMethodNotification->deactivate();
                        last;
                    }
                }
            }
        };
        warn('Patch failed in shop ' . $ShopID . " with error \"" . GetError() . "\".") if ExistsError();
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { InitializeSetUpExternalPaymentNotification(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Adds the set up external payment notification for all Unity shops.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
