#========================================================================================
# package      DE_EPAGES::Product::API::ProductImageService
# state        public
#----------------------------------------------------------------------------------------
# description  Package that provides methods for managing product images.
#========================================================================================
package DE_EPAGES::Product::API::ProductImageService;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( CleanPath DeleteFile WriteFile fsplit );
use DE_EPAGES::Core::API::Image qw ( CorrectImage TestImage TestImageFormat );
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Filename );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Product::API::Constants qw ( PRODUCT_IMAGE_SIZE_NAMES PRODUCT_IMAGES_SIZES);
use DE_EPAGES::Product::API::Product qw ( ScaleImageSizes DeleteProductImage );
use DE_EPAGES::SEO::API::SEO qw ( PerformSEOCheckAndUpdateObject );
use List::Util qw ( any );
use Image::Magick;

use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Product );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# input        $Product | Product | DE_EPAGES::Product::API::Object::Product
# return       $self | self | DE_EPAGES::Product::API::ProductImageService
#========================================================================================
sub new {
    my $class = shift;
    my ($Product) = @_;
    my $self  = bless {
        'Product' => $Product,
    }, $class;
    return $self;
}



sub uploadAndScaleImage {
    my $self = shift;
    my ($FileName, $Content) = @_;

    my $Product = $self->{'Product'};

    my $hUploadedFileInfo = $self->_saveFile($Product, $FileName, $Content);
    my $ShopPath = $Product->get('Site')->get('PublicPath');
    my $ProductPath = $Product->get('FileAlias');

    my $PathOfUploadedFileInFileSystem = $hUploadedFileInfo->{'FileSystemPath'};

    eval {
        TestImage($PathOfUploadedFileInFileSystem);
        my $hScaleParams = $self->_createScaleParameters();
        ScaleImageSizes($PathOfUploadedFileInFileSystem, $hScaleParams);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( $Error->code() ~~ [qw ( IMAGE_MAGICK UNSUPPORTED_IMAGE_FORMAT INVALID_IMAGE_FILE IMAGE_EXCEEDS_MEGAPIXELS )] ) {
            DeleteFile($PathOfUploadedFileInFileSystem);
            CleanPath($ShopPath, $ProductPath);
        } else {
            Error($Error);
        }
    }

    return $hUploadedFileInfo;
}


#========================================================================================
# function     updateSlideShowString
# state        public
#----------------------------------------------------------------------------------------
# syntax       prependSlideShowString( $FileName );
#----------------------------------------------------------------------------------------
# description  updates ImageSlideShowString, puts filename at the end of the slideshow
#----------------------------------------------------------------------------------------
# input        $ImmageLarge | name of file | string
#========================================================================================
sub updateSlideShowString {
    my $self = shift;
    my ($FileName) = @_;

    my $Product = $self->{'Product'};
    my $SlideShowString = $Product->get('ImagesSlideShowString');
    my $NewFileRel = Unicode2Filename($FileName);

    if(!$SlideShowString) {
        my $DefaultImage = $Product->get('ImageLarge');
        $SlideShowString = defined $DefaultImage ? $DefaultImage . ';' . $NewFileRel : $NewFileRel;
        $Product->set( { 'ImagesSlideShowString' => $SlideShowString } );
    } elsif($SlideShowString !~ /$NewFileRel/) {
        $SlideShowString .= ';'.$NewFileRel;
        $Product->set( { 'ImagesSlideShowString' => $SlideShowString } );
    }

    return;
}

#========================================================================================
# function     prependSlideShowString
# state        public
#----------------------------------------------------------------------------------------
# syntax       prependSlideShowString( $FileName );
#----------------------------------------------------------------------------------------
# description  updates ImageSlideShowString, puts filename at the front of the slideshow
#----------------------------------------------------------------------------------------
# input        $ImmageLarge | name of file  | string
#========================================================================================
sub prependSlideShowString {
    my $self = shift;
    my ($FileName) = @_;

    my $Product = $self->{'Product'};
    my $SlideShowString = $Product->get('ImagesSlideShowString');
    my $NewFileRel = Unicode2Filename($FileName);

    if(!$SlideShowString) {
        my $DefaultImage = $Product->get('ImageLarge');
        $SlideShowString = defined $DefaultImage ? $DefaultImage . ';' . $NewFileRel : $NewFileRel;
        $Product->set( { 'ImagesSlideShowString' => $SlideShowString } );
    } elsif($SlideShowString !~ /$NewFileRel/) {
        $SlideShowString = $NewFileRel . ';' . $SlideShowString;
        $Product->set( { 'ImagesSlideShowString' => $SlideShowString } );
    }

    return;
}



sub setSlideShowImageAsProductImage {
    my $self = shift;
    my ($ImageName) = @_;

    my $Product = $self->{'Product'};
    my ( undef, $FName, $Extension ) = fsplit($ImageName);

    # Check if imageName exists in ImageSlideShow
    my $aImages = $Product->get('ImagesSlideShow');
    my $ExistsImage = any {$ImageName eq $_->{'ImageName'}} @$aImages;

    if ( !$ExistsImage ) {
        Error('ResourceNotFound');
    }

    # use original name for large image
    $Product->set( { 'ImageLarge' => $ImageName } );

    # now let us consider all other sizes except 'ImageLarge'
    my @ImagesSizeSuffixes = keys %{ PRODUCT_IMAGE_SIZE_NAMES() };
    my @ImagesSizeSuffixesExceptImageLarge = grep { PRODUCT_IMAGE_SIZE_NAMES->{$_} ne 'ImageLarge'} @ImagesSizeSuffixes;

    # for all product image sizes we add '_' and 'image size suffix' in the end of image name and before '.extension'
    foreach my $ImageSizeSuffix (@ImagesSizeSuffixesExceptImageLarge) {
        my $ImageSizeName = PRODUCT_IMAGE_SIZE_NAMES->{$ImageSizeSuffix};
        $Product->set({
            $ImageSizeName => $FName . '_' . $ImageSizeSuffix . $Extension
        });
    }

    $Product->updatePageCache();
    $self->_PerformSEOCheckAndUpdateObject($Product);

    return;
}



sub deleteSlideShowImage {
    my $self = shift;
    my ($ImageName) = @_;

    my $Product = $self->{'Product'};
    $self->_checkIfImageExistsInSlideShow($ImageName);

    $self->_DeleteProductImage($Product, $ImageName);

    return;
}



sub _checkIfImageExistsInSlideShow {
    my $self = shift;
    my ($ImageName) = @_;

    my $Product = $self->{'Product'};
    my $aImages = $Product->get('ImagesSlideShow');
    my $ExistsImage = any {$ImageName eq $_->{'ImageName'}} @$aImages;

    if ( !$ExistsImage ) {
        Error('ResourceNotFound');
    }

    return;
}



sub _DeleteProductImage {
    my $self = shift;
    my ($Product, $ImageName) = @_;
    DeleteProductImage($Product, $ImageName);
    return;
}



sub _PerformSEOCheckAndUpdateObject {
    my $self = shift;
    my ($Product) = @_;
    PerformSEOCheckAndUpdateObject($Product);
    return;
}



sub _createScaleParameters {
    my $self = shift;

    my $hParams = {};
    my @Sizes = keys %{ PRODUCT_IMAGE_SIZE_NAMES() };
    for my $Size (@Sizes) {
        $hParams->{'SCALE_NAME_'.$Size} = PRODUCT_IMAGE_SIZE_NAMES->{$Size};
        $hParams->{'SCALE_WIDTH_'.$Size} = PRODUCT_IMAGES_SIZES->{$Size}[0];
        $hParams->{'SCALE_HEIGHT_'.$Size} = PRODUCT_IMAGES_SIZES->{$Size}[1];
        $hParams->{'SCALE_QUALITY_'.$Size} = 80; ## no critic (ProhibitMagicNumbers)
    }

    return $hParams;
}



sub _saveFile {
    my $self = shift;
    my ( $Product , $NewFileName, $Content ) = @_;

    my $Shop = $Product->getSite();

    # Save file in 'PublicPath' which is the default file system directory for uploads.
    my $FileSystemDirectory = $Product->get('PublicPath');
    $FileSystemDirectory .= '/' if $FileSystemDirectory !~ /\/$/;

    # Options for WriteFile.
    my $Binary          = 1;
    my $CreateDirectory = 1;

    my $NameOfUploadedFile = $self->_makeUpload($Product, $Shop, $Content, $FileSystemDirectory, $Binary, $CreateDirectory, $NewFileName);

    my $PublicWebDirectory = $Product->get('WebPath');
    $PublicWebDirectory .= '/' if $PublicWebDirectory !~ /\/$/;

    my $hFileInfo = {
        'WebPath' => $PublicWebDirectory . $NameOfUploadedFile,
        'FileSystemPath' => $FileSystemDirectory . $NameOfUploadedFile,
        'FileName' => $NameOfUploadedFile,
    };

    return $hFileInfo;
}



sub _makeUpload {
    my $self = shift;
    my ($Product, $Shop, $Content, $Directory, $Binary, $CreateDirectory, $NewFileName) = @_;

    #Get the filename and extension.
    my( undef, $FName, $Extension ) = fsplit( $NewFileName );
    my $OldFileName = $FName;
    my $OldExtension = $Extension;

    # Check if there is already a '.' ahead the extension.
    $Extension = '.'.$Extension unless ($Extension =~ /^\./);
    my $FileName = Unicode2Filename($FName) . $Extension;

    # The complete filepath on the server.
    $FName = $Directory . $FileName;

    # Trigger Hook (e.q. Virenscan)
    TriggerHook('UI_BeforeUploadWriteFile', {
        'Shop'              => $Shop,
        'FileName'          => $OldFileName.$OldExtension,
        'FileContent'       => \$Content,
    });

    # Write file temporary.
    my $TempFile = $FName;
    WriteFile( $TempFile, \$Content, {
        'binary' => $Binary,
        'create_directory' => $CreateDirectory
    });

    # Check if file is image file.
    eval {
        TestImageFormat('UploadedImage', $TempFile);
    };
    if(!ExistsError()) {
        # Check if image is corrupt.
        eval {
            TestImage($TempFile);
        };
        if(ExistsError()) {
            DeleteFile($TempFile);
            my $Error = GetError();
            Error($Error->code);
        }
        # to RGB
        CorrectImage($TempFile);
    }

    # Trigger Hook (e.q. statistic)
    TriggerHook('UI_AfterUploadWriteFile',{
        'Shop' => $Shop,
        'FileNameCustomer' => $OldFileName.$OldExtension,
        'FileNameEPages' => $FileName,
        'FilePath' => $Directory
    });

    return $FileName;
}

1;