#========================================================================================
# package      DE_EPAGES::RemoteSearch::Hooks::ContentObject
# state        private
#----------------------------------------------------------------------------------------
# description  Keep track of changes to categories
#========================================================================================
package DE_EPAGES::RemoteSearch::Hooks::ContentObject;

use strict;

use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::RemoteSearch::API::AttributeModifiedCheck qw ( RunIfModified );
use DE_EPAGES::RemoteSearch::API::Batch qw ( BatchUpdateProduct IsScheduleMode );
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw ( RunIfRemoteSearchInitialized MarkShopDirty );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient;

my $hUpdateCategory = {};

sub OnBeforeUpdateContentObject {
    my ($hContent) = @_;
    my $Category = $hContent->{Object};

    return unless $Category->instanceOf('Category');
    my $hValues = $hContent->{'Values'};

    my @LOCALIZED     = qw ( NavigationCaption Keywords Name );
    my @LOC_TYPES     = qw ( String String String );
    my @NON_LOCALIZED = qw ( Alias Parent IsVisible );
    my @NON_LOC_TYPES = qw ( String Object Boolean );

    my $DO_UPDATE = 0;
    RunIfModified(
        $Category,
        \@LOCALIZED,
        $hValues,
        $hContent->{'LanguageID'},
        \@LOC_TYPES,
        sub {
            $DO_UPDATE = 1;
        }
    );

    RunIfModified(
        $Category,
        \@NON_LOCALIZED,
        $hValues, undef,
        \@NON_LOC_TYPES,
        sub {
            $DO_UPDATE = 1;
        }
    );

    if($DO_UPDATE) {
        $hUpdateCategory->{$Category->guid()} = 1;
    }

    return;
}


sub OnAfterUpdateContentObject{
    my($hContent) = @_;
    my $Category = $hContent->{Object};

    return unless $Category->instanceOf('Category');
    my $Shop = $Category->getSite();

    #AD-7352: This function used to be on the beforeUpdate, but due some sync problems it had to be moved to here
    return unless $hUpdateCategory->{$Category->guid()};

    _UpdateShop($Shop, $Category);
    delete $hUpdateCategory->{$Category->guid()};# this deletes only the key/value of the hash

    return;
}

sub OnDeleteContentObject {
    my ($hContent) = @_;
    my $Category = $hContent->{Object};
    my $Shop = $Category->getSite();
    return unless $Category->instanceOf('Category');
    _UpdateShop($Shop, $Category);
    return;
}

sub OnInsertContentObject {
    my ($hContent) = @_;

    my $Category = $hContent->{'Object'};

    return unless $Category->instanceOf('Category');

    GetLog->debug('New category insertion');
    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->create($Category);
    $Client->createCategory($Category);

    return;
}


# If Category is a leaf update Products, else reindex shop
sub _UpdateShop {
    my($Shop, $Category) = @_;

    if ( IsScheduleMode() ) {
        MarkShopDirty($Shop);
        return;
    }

    my $aSubCategories = $Category->get('SubCategories');

    if ( scalar @$aSubCategories > 0 ) {
        GetLog->debug('Category is not a leaf - reindex Shop');
        RunIfRemoteSearchInitialized($Shop, sub {
            my ($Search) = @_;
            $Search->initialize();
        });
    }
    else {
        GetLog->debug('Category is a leaf - update Products');
        RunBatchProcess('UpdateProducts', sub{
            BatchUpdateProduct($_) foreach(@{$Category->get('VisibleProducts')});
        });
    }
}

1;
