#========================================================================================
# package      DE_EPAGES::SIA::API::Object::PaymentMethodSIA
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerPaymentMethod
#----------------------------------------------------------------------------------------
# description  object interface for Payment.
#========================================================================================
package DE_EPAGES::SIA::API::Object::PaymentMethodSIA;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;


use DE_EPAGES::Object::API::Factory         qw (Transaction);
use DE_EPAGES::SIA::API::Constants          qw (INTERNAL_CARTRIDGE_NAME TRANS_STATUS_PENDING TRANS_STATUS_ACCEPTED TRANS_STATUS_DECLINED
                                                TIMESTAMPTIME ADYENACCOUNT RSAVERIFICATION);
use DE_EPAGES::SIA::API::Tools              qw (GetToolsInstance);
use DE_EPAGES::Core::API::PerlTools         qw (GenerateGUID isValid);
use DE_EPAGES::Object::API::Table::Language qw (GetInfoLanguage);
use Digest::SHA1                            qw (sha1_hex);
use DE_EPAGES::SIA::API::SIAConfig;

use strict;


#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethod->featureName;
#----------------------------------------------------------------------------------------
# description   Returns the feature name 'SIA'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName {
    return INTERNAL_CARTRIDGE_NAME;
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a payment method is visible in the storefront.
#               Overrides function <function DE_EPAGES::Payment::API::Object::PaymentMethod::usableAtStorefront>
#               to evaluate if payment method is configured
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;
    my $IsUsableAtStorefront = 0;

    return $IsUsableAtStorefront unless $self->existsCurrentRegistrationTransaction();

    my $RegistrationData = $self->getCurrentRegistrationTransaction();

    if ($RegistrationData->get('ApplicationPaymentNetwork')     &&
        $RegistrationData->get('MerchantID')     &&
        $RegistrationData->get('TerminalNumber') &&
        $RegistrationData->get('MerchantTaxID')  &&
        $self->SUPER::usableAtStorefront($hVars) &&
        ( $RegistrationData->get('TransStatus') eq TRANS_STATUS_ACCEPTED || $self->isInTestMode() )) {

        $IsUsableAtStorefront = 1;
    }

    return $IsUsableAtStorefront;
}

#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars)) {...
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               Valid currencies are: EUR, GBP
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    return $self->SUPER::canAddToBasket($hVars);
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::SIA::API::Object::LineItemPaymentSIA->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ( $Parent, $hInfo ) = @_;

    return { %{ $class->SUPER::default( $Parent, $hInfo ) }, 'IsInTestMode' => 1 };
}

#========================================================================================
# function     getSaleRedirectURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->getSaleRedirectURL;
#----------------------------------------------------------------------------------------
# description  returns the sale redirect url for the payment
#----------------------------------------------------------------------------------------
# return        $SaleWebserviceEndpointURL | sale redirect url | string
#========================================================================================
sub getSaleRedirectURL {
    my $self = shift;

    return $self->getConfig()->getSaleWebserviceEndpointURL;
}

#========================================================================================
# function     getRefundRedirectURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->getRefundRedirectURL;
#----------------------------------------------------------------------------------------
# description  returns the refund redirect url for the payment
#----------------------------------------------------------------------------------------
# return        $RefundWebserviceEndpointURL | refund rest url | string
#========================================================================================
sub getRefundRedirectURL {
    my $self = shift;

    return $self->getConfig()->getRefundWebserviceEndpointURL;
}

#========================================================================================
# function     getCancelRedirectURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->getCancelRedirectURL;
#----------------------------------------------------------------------------------------
# description  returns the cancel redirect url for the payment
#----------------------------------------------------------------------------------------
# return        $CancelWebserviceEndpointURL | cancel rest url | string
#========================================================================================
sub getCancelRedirectURL {
    my $self = shift;

    return $self->getConfig()->getCancelWebserviceEndpointURL;
}

#========================================================================================
# function     isInTestMode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->isInTestMode;
#----------------------------------------------------------------------------------------
# description  returns if payment is in test mode
#----------------------------------------------------------------------------------------
# return        $isInTestMode | ispayment in testmode | boolean
#========================================================================================
sub isInTestMode {
    my $self = shift;

    return $self->get('IsInTestMode');
}

#========================================================================================
# function     getConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->getConfig;
#----------------------------------------------------------------------------------------
# description  createa a new siaonfig object dependent from payment testmode
#----------------------------------------------------------------------------------------
# return        $SiaConfig | sia config | object
#========================================================================================
sub getConfig {
    my $self = shift;

    return DE_EPAGES::SIA::API::SIAConfig->new( 'IsInTestMode' => $self->isInTestMode() );
}

#========================================================================================
# function     saveRegistrationTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->saveRegistrationTransaction;
#----------------------------------------------------------------------------------------
# description  saves a new registration transaction on payment and sets it as current transaction
#----------------------------------------------------------------------------------------
# input        $hRegistrationTransaction | registration transaction | ref.hash
#----------------------------------------------------------------------------------------
# return       $RegistrationTransaction | registration transaction | object
#========================================================================================
sub saveRegistrationTransaction {
    my $self = shift;
    my ($hRegistrationTransaction) = @_;

    my $RegistrationTransactions = $self->folder('RegistrationTransactions');
    my $RegistrationTransaction;

    $self->_transaction(
        sub {
            $RegistrationTransaction = $RegistrationTransactions->insertChild( 'SIARegistrationTransaction', $hRegistrationTransaction );
            $self->set( { 'CurrentRegistrationTransaction' => $hRegistrationTransaction->{'Alias'} } );
            $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_PENDING});
        }
    );

    return $RegistrationTransaction;
}

#========================================================================================
# function     hasPendingCurrentTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->hasPendingCurrentTransaction;
#----------------------------------------------------------------------------------------
# description  returns true if the current registration transaction is still pending
#----------------------------------------------------------------------------------------
# return       hasPendingCurrentTransaction | has pending current transaction | boolean
#========================================================================================
sub hasPendingCurrentTransaction {
    my $self = shift;

    my $RegistrationTransaction = $self->getCurrentRegistrationTransaction();

    return 0 unless defined $RegistrationTransaction;
    return 1 if $RegistrationTransaction->get('TransStatus') eq TRANS_STATUS_PENDING;

    return 0;
}

#========================================================================================
# function     hasAcceptedCurrentTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->hasAcceptedCurrentTransaction;
#----------------------------------------------------------------------------------------
# description  returns true if the current registration transaction is accepted
#----------------------------------------------------------------------------------------
# return       hasAcceptedCurrentTransaction | has accepted current transaction | boolean
#========================================================================================
sub hasAcceptedCurrentTransaction {
    my $self = shift;

    my $RegistrationTransaction = $self->getCurrentRegistrationTransaction();

    return 0 unless defined $RegistrationTransaction;
    return 1 if $RegistrationTransaction->get('TransStatus') eq TRANS_STATUS_ACCEPTED;

    return 0;
}


#========================================================================================
# function     hasDeclinedCurrentTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->hasDeclinedCurrentTransaction;
#----------------------------------------------------------------------------------------
# description  returns true if the current registration transaction is declined
#----------------------------------------------------------------------------------------
# return       hasDeclinedCurrentTransaction | has declined current transaction | boolean
#========================================================================================
sub hasDeclinedCurrentTransaction {
    my $self = shift;

    my $RegistrationTransaction = $self->getCurrentRegistrationTransaction();

    return 0 unless defined $RegistrationTransaction;
    return 1 if $RegistrationTransaction->get('TransStatus') eq TRANS_STATUS_DECLINED;

    return 0;
}


#========================================================================================
# function     existsCurrentRegistrationTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->existsCurrentRegistrationTransaction;
#----------------------------------------------------------------------------------------
# description  exists a registration transation for the paymentmethod
#----------------------------------------------------------------------------------------
# return        $Exists | exists current registration transaction | boolean
#========================================================================================
sub existsCurrentRegistrationTransaction {
    my $self = shift;

    my $RegistrationTransaction = $self->getCurrentRegistrationTransaction();

    return 0 unless defined $RegistrationTransaction;
    return 1;
}

#========================================================================================
# function     getCurrentRegistrationTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->getCurrentRegistrationTransaction;
#----------------------------------------------------------------------------------------
# description  gives back the current registration transaction which is used for the merchant registration
#----------------------------------------------------------------------------------------
# return       $Transaction | actual merchant registration transaction | object
#========================================================================================
sub getCurrentRegistrationTransaction {
    my $self = shift;

    my $TransactionAlias         = $self->get('CurrentRegistrationTransaction');
    my $RegistrationTransactions = $self->folder('RegistrationTransactions');

    return undef unless isValid($TransactionAlias);
    return undef unless $RegistrationTransactions->existsChild($TransactionAlias);

    return $RegistrationTransactions->child($TransactionAlias);
}

#========================================================================================
# function     provideRegistrationTransactionData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod->provideRegistrationTransactionData( $PaymentNetwork,
#                                                                   $MerchantID,
#                                                                   $TerminalNumber,
#                                                                   $MerchantTaxID);
# example       $PaymentMethod->provideRegistrationTransactionData('Adyen','123123123123123','12312312','9486948');
#----------------------------------------------------------------------------------------
# description  provide the values which are needed for merchant registration
#----------------------------------------------------------------------------------------
# input        $PaymentNetwork             | payment network which was entered by merchant     | Int
# input        $MerchantID                 | merchant id which was entered by merchant         | String
# input        $TerminalNumber             | terminal number id which was entered by merchant  | String
# input        $MerchantTaxID              | merchant tax id which was entered by merchant     | String
# return       $hRegistrationTransaction   | dynamic values for emrchant registration          | hash.ref
#========================================================================================
sub provideRegistrationTransactionData {
    my $self = shift;
    my ($PaymentNetwork, $MerchantID, $TerminalNumber, $MerchantTaxID, $ThreeDSecure) = @_;

    my $Shop        = $self->getSite();
    my $ShopAddress = $Shop->get('Address');
    my $SiaConfig   = $self->getConfig();
    my $hRegistrationTransaction;

    $hRegistrationTransaction->{'Alias'}                     = GenerateGUID();

    $hRegistrationTransaction->{'ApplicationPaymentNetwork'} = $PaymentNetwork;
    $hRegistrationTransaction->{'MerchantID'}                = $MerchantID;
    $hRegistrationTransaction->{'TerminalNumber'}            = $TerminalNumber;
    $hRegistrationTransaction->{'MerchantTaxID'}             = $MerchantTaxID;

    # merchant fuc needs to start with 004 for acens (also 15 digits long like merchantid, truncat the rest)
    $hRegistrationTransaction->{'MerchantFUC'}               = $SiaConfig->getMerchantFUCPrefix().sha1_hex($MerchantID);
    $hRegistrationTransaction->{'WorkOrderID'}               = sha1_hex(GenerateGUID().$SiaConfig->getIMCompany());

    $hRegistrationTransaction->{'MerchantName'}              = $ShopAddress->get('FullName');
    $hRegistrationTransaction->{'City'}                      = $ShopAddress->get('City');
    $hRegistrationTransaction->{'Country'}                   = $ShopAddress->get('CountryID');

    $hRegistrationTransaction->{'WorkOrderDate'}             = GetToolsInstance()->getCurrentDate();
    $hRegistrationTransaction->{'MerchantLanguage'}          = $self->_getInfoLanguage($Shop->get('LanguageID'));
    $hRegistrationTransaction->{'Currency'}                  = $Shop->get('CurrencyID');

    $hRegistrationTransaction->{'IMCompany'}                 = $SiaConfig->getIMCompany();
    $hRegistrationTransaction->{'CompanyCode'}               = $SiaConfig->getCompanyCode();
    $hRegistrationTransaction->{'HoldingCode'}               = $SiaConfig->getHoldingCode();
    $hRegistrationTransaction->{'AcquiringFinancialEntity'}  = $SiaConfig->getAcquiringFinancialEntity();
    $hRegistrationTransaction->{'AcquiringMerchant'}         = $MerchantID;
    $hRegistrationTransaction->{'TimeStampTime'}             = TIMESTAMPTIME;
    $hRegistrationTransaction->{'RsaVerification'}           = RSAVERIFICATION;

    # add this field if its a adyen account
    $hRegistrationTransaction->{'AdyenAccount'}              = ADYENACCOUNT if $PaymentNetwork eq '110001';
    $hRegistrationTransaction->{'ThreeDSecureConfiguration'} = "01" if $ThreeDSecure;

    return $hRegistrationTransaction;
}

sub _transaction {
    my $self = shift;
    my ($cCode) = @_;

    Transaction($cCode);

    return;
}

sub _getInfoLanguage {
    my $self = shift;
    my ($LanguageID) = @_;

    return GetInfoLanguage($LanguageID)->{'Code2'};
}

1;
