#========================================================================================
# package      DE_EPAGES::Shipping::API::Object::ShippingType
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for ShippingType.
#========================================================================================
package DE_EPAGES::Shipping::API::Object::ShippingType;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Object::API::Language qw ( GetCurrentLanguageID );
use DE_EPAGES::Shipping::API::Shop qw ( GetShippingMethodsFolder );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ShippingType->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ShippingType' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ShippingType->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingType = DE_EPAGES::Shipping::API::Object::ShippingType->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShippingType');
#               my $ShippingType = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertShippingType' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ShippingType | new object | object
# hook         OBJ_InsertShippingType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingTypeID - shipping type object id - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertShippingType', {'Object' => $self, 'ShippingTypeID' => $self->id });
    delete $hInfo->{$_} foreach qw ( MethodClass LineItemClass IsActivated CanRegions ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingType->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteShippingType' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShippingType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingTypeID - shipping type object id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteShippingType', {'Object' => $self, 'ShippingTypeID' => $self->id });
    LoadClassByAlias('ShippingMethod')->deleteObjects('ByShippingTypeID', [$self->id]);
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingType->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShippingType' and 'OBJ_AfterUpdateShippingType'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShippingType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingTypeID - shipping type object id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShippingType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingTypeID - shipping type object id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateShippingType', {'Object'=>$self, 'Values'=> $hValues, 'ShippingTypeID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShippingType',  {'Object'=>$self, 'Values'=> $hValues, 'ShippingTypeID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

sub importXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;

    $hAttributes->{'MethodClass'} = LoadClassByAlias($hAttributes->{'MethodClass'}) if defined $hAttributes->{'MethodClass'};
    $hAttributes->{'LineItemClass'} = LoadClassByAlias($hAttributes->{'LineItemClass'}) if defined $hAttributes->{'LineItemClass'};
    return $ObjectPackage->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);
}

#========================================================================================
# function     exportXMLAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes bit attributes with value 0.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function exportableAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;
    $hAttributes = $self->SUPER::exportXMLAttributes($Driver, $hAttributes);
    my $MethodClass = $self->get('MethodClass');
    $hAttributes->{'MethodClass'} = $MethodClass->alias if defined $MethodClass;

    my $LineItemClass = $self->get('LineItemClass');
    $hAttributes->{'LineItemClass'} = $LineItemClass->alias if defined $LineItemClass;

    return $hAttributes;
}

#========================================================================================
# function     newShippingMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ShippingMethod = $ShippingType->newShippingMethod($Shop, $hInfo);
#----------------------------------------------------------------------------------------
# description  Creates a new shipping method with the given parameters.
#               Initializes the attribute 'Name' if the method is called with a language
#               context, for example, when called from an UI handler.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $hInfo | object attributes for new object, e.g. Alias | ref.hash
# return       $ShippingMethod | new object | DE_EPAGES::Shipping::API::Object::ShippingMethod
#========================================================================================
sub newShippingMethod {
    my $self = shift;
    my ($Shop, $hInfo) = @_;

    my $Folder = GetShippingMethodsFolder($Shop);  # position is necessary for export
    $hInfo->{'ShippingType'} = $self;
    $hInfo->{'Alias'} = $Folder->nextAlias('ShippingMethod%4d') unless isValid( $hInfo->{'Alias'} ) ;
    my $MethodClass = $self->get('MethodClass');
    my $ClassName = defined $MethodClass ? $MethodClass->alias : 'ShippingMethod';
    my $ShippingMethod = $Folder->insertChild($ClassName, $hInfo);

    # copy the alias to the Name attribute if there is a language context is provided
    my $LanguageID = GetCurrentLanguageID();
    $ShippingMethod->set({ 'Name' => $hInfo->{'Alias'} }, $LanguageID ) if defined $LanguageID;

    # set name for all languages in a Unity Shop
    if ($Shop->get('IsUnityRegistered')) {
        my @ShopLanguageIDs = map { $_->{'LanguageID'} } @{$Shop->get('Languages')};
        if (scalar @ShopLanguageIDs > 1) {
            foreach my $ShopLanguageID (@ShopLanguageIDs) {
                next if $ShopLanguageID == $LanguageID;
                $ShippingMethod->set({'Name' => $hInfo->{'Alias'}}, $ShopLanguageID);
            }
        }
    }

    return $ShippingMethod;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Shipping::API::Object::ShippingType->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'MethodClass' => undef,
        'LineItemClass' => undef,
        'IsActivated' => 1,
        'CanRegions' => 1,
    };
}

1;
