#========================================================================================
# package      DE_EPAGES::Shop::API::ShopLanguage
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the shop language functions.
#========================================================================================
package DE_EPAGES::Shop::API::ShopLanguage;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    PresetUnityShopObjectNames
    SetNameForNewLanguage
);

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestString );

sub ErrorDefinition {
    return {
        'AttributeDoesNotExist' => 'Attribute \'#AttributeAlias\' does not exist for class \'Shop\'',
        'NoArrayAttribute' => 'Attribute \'#AttributeAlias\' is not an array',
    };
}


#========================================================================================
# function     PresetUnityShopObjectNames
# state        public
#----------------------------------------------------------------------------------------
# syntax       PresetUnityShopObjectNames($ShopLanguage, $ShopObjectFolderName);
#----------------------------------------------------------------------------------------
# example      PresetUnityShopObjectNames($hParams->{'Object'}, 'PaymentMethods');
#----------------------------------------------------------------------------------------
# description  Sets the names of all objects in a given object folder of a Unity shop
#               for the given language with the existing value of the shop's default
#               language
#----------------------------------------------------------------------------------------
# input        $ShopLanguage | shop language | DE_EPAGES::Shop::API::Object::ShopLanguage
# input        $ShopObjectFolderName | object folder name | string
#========================================================================================
sub PresetUnityShopObjectNames {
    my ($ShopLanguage, $ShopObjectFolderName) = @_;

    TestObject('ShopLanguage', $ShopLanguage, 'DE_EPAGES::Shop::API::Object::ShopLanguage');
    TestString('ShopObjectFolderName', $ShopObjectFolderName);

    my $Shop = $ShopLanguage->getSite();

    # do this only for Unity shops
    if ($Shop->get('IsUnityRegistered')) {
        my $ShopClass = $Shop->class;
        # check if attribute exists
        if ($ShopClass->existsAttribute($ShopObjectFolderName)) {
            # check if we really have an array attribute
            if ($ShopClass->attribute($ShopObjectFolderName)->isArray) {
                my $aObjectFolder = $Shop->get($ShopObjectFolderName);
                my $NewLanguageID = $ShopLanguage->get('LanguageID');
                my $DefaultLanguageID = $Shop->get('LanguageID');
                # set names in the new language
                foreach my $Object (@{$aObjectFolder}) {
                    SetNameForNewLanguage($Object, $DefaultLanguageID, $NewLanguageID);
                }
            } else {
                Error('NoArrayAttribute', { 'AttributeAlias' => $ShopObjectFolderName });
            }
        } else {
            Error('AttributeDoesNotExist', { 'AttributeAlias' => $ShopObjectFolderName });
        }
    }

    return;
}


#========================================================================================
# function     SetNameForNewLanguage
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetNameForNewLanguage($Object, $DefaultLanguageID, $NewLanguageID);
#----------------------------------------------------------------------------------------
# example      SetNameForNewLanguage($Product, $DefaultLanguageID, $NewLanguageID);
#----------------------------------------------------------------------------------------
# description  Sets the name of an object in a new language with the name in the default
#               language
#----------------------------------------------------------------------------------------
# input        $Object | object | Object
# input        $DefaultLanguageID | language id | integer
# input        $NewLanguageID | language id | integer
#========================================================================================
sub SetNameForNewLanguage {
    my ($Object, $DefaultLanguageID, $NewLanguageID) = @_;

    unless (defined $Object->get('Name', $NewLanguageID)) {
        my $Name = $Object->get('Name', $DefaultLanguageID);
        $Object->set({'Name' => $Name}, $NewLanguageID) if defined $Name;
    }

    return;
}

1;
