# test for the API function DE_EPAGES::Shop::API::ShopLanguage::PresetUnityShopObjectNames

package DE_EPAGES::Shop::t::ShopLanguage;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Shop::API::ShopLanguage qw ( PresetUnityShopObjectNames );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Exception;
use Test::More;
use strict;

use constant LANGUAGE_ID_DE => 1;
use constant LANGUAGE_ID_EN => 2;
use constant PAYMENT_METHODS => 'PaymentMethods';
use constant WRONG_PAYMENT_METHODS => 'PaymntMethods';
use constant IS_UNITY_REGISTERED => 'IsUnityRegistered';

sub test {
    my $self = shift;
    my $UnityShop = $self->_createMockedShop(1);
    my $NoUnityShop = $self->_createMockedShop(0);
    my $UnityShopLanguage = $self->_createMockedShopLanguage($UnityShop);
    my $NoUnityShopLanguage = $self->_createMockedShopLanguage($NoUnityShop);

    note('Parameter tests');
    throws_ok( sub {PresetUnityShopObjectNames($NoUnityShop, PAYMENT_METHODS)}, qr/WRONG_CLASS/,
        "call PresetUnityShopObjectNames not with 'ShopLanguage' object --> error");
    throws_ok( sub {PresetUnityShopObjectNames($NoUnityShopLanguage, $NoUnityShop)}, qr/STRING_ISOBJECT/,
        "call PresetUnityShopObjectNames without String parameter --> error");

    note('No Unity shop');
    my $PaymentMethods = $NoUnityShop->get(PAYMENT_METHODS);
    is( $PaymentMethods->[0]->get('Name', LANGUAGE_ID_EN), undef, "Englisch name has not been set for payment method 1");
    is( $PaymentMethods->[1]->get('Name', LANGUAGE_ID_EN), undef, "Englisch name has not been set for payment method 2");
    is( $PaymentMethods->[2]->get('Name', LANGUAGE_ID_EN), undef, "Englisch name has not been set for payment method 3");

    note('call PresetUnityShopObjectNames');
    PresetUnityShopObjectNames($NoUnityShopLanguage, PAYMENT_METHODS);
    $PaymentMethods = $NoUnityShop->get(PAYMENT_METHODS);
    is( $PaymentMethods->[0]->get('Name', LANGUAGE_ID_EN), undef, "Englisch name is still not set for payment method 1");
    is( $PaymentMethods->[1]->get('Name', LANGUAGE_ID_EN), undef, "Englisch name is still not set for payment method 2");
    is( $PaymentMethods->[2]->get('Name', LANGUAGE_ID_EN), undef, "Englisch name is still not set for payment method 3");

    note('Unity shop');
    $PaymentMethods = $UnityShop->get(PAYMENT_METHODS);
    is( $PaymentMethods->[0]->get('Name', LANGUAGE_ID_EN), undef, "Englisch name has not been set for payment method 1");
    is( $PaymentMethods->[1]->get('Name', LANGUAGE_ID_EN), undef, "Englisch name has not been set for payment method 2");
    is( $PaymentMethods->[2]->get('Name', LANGUAGE_ID_DE), undef, "German name has not been set for payment method 3");
    is( $PaymentMethods->[2]->get('Name', LANGUAGE_ID_EN), undef, "Englisch name has not been set for payment method 3");

    throws_ok( sub {PresetUnityShopObjectNames($UnityShopLanguage, WRONG_PAYMENT_METHODS)}, qr/AttributeDoesNotExist/,
        "call PresetUnityShopObjectNames with wrong folder name --> error");
    throws_ok( sub {PresetUnityShopObjectNames($UnityShopLanguage, IS_UNITY_REGISTERED)}, qr/NoArrayAttribute/,
        "call PresetUnityShopObjectNames with non-array attribute --> error");

    note('call PresetUnityShopObjectNames with attribute' . PAYMENT_METHODS);
    PresetUnityShopObjectNames($UnityShopLanguage, PAYMENT_METHODS);
    ok( defined $PaymentMethods->[0]->get('Name', LANGUAGE_ID_EN), "Englisch name is set for payment method 1");
    is( $PaymentMethods->[0]->get('Name', LANGUAGE_ID_EN), $PaymentMethods->[0]->get('Name', LANGUAGE_ID_DE),
        "Englisch name of payment method 1 equals the German name");
    ok( defined $PaymentMethods->[1]->get('Name', LANGUAGE_ID_EN), "Englisch name is set for payment method 2");
    is( $PaymentMethods->[1]->get('Name', LANGUAGE_ID_EN), $PaymentMethods->[1]->get('Name', LANGUAGE_ID_DE),
        "Englisch name of payment method 2 equals the German name");
    ok( !defined $PaymentMethods->[2]->get('Name', LANGUAGE_ID_EN), "Englisch name is not set for payment method 3");
    is( $PaymentMethods->[2]->get('Name', LANGUAGE_ID_EN), $PaymentMethods->[2]->get('Name', LANGUAGE_ID_DE),
        "Englisch name of payment method 3 equals the German name");

    done_testing();
}

sub _createMockedShopLanguage {
    my $self = shift;
    my ($Shop) = @_;

    my $self = shift;
    my $PackageName = 'DE_EPAGES::Shop::API::Object::ShopLanguage';
    my $hAttributes = {
        'LanguageID' => LANGUAGE_ID_EN,
        'Site'       => $Shop
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, $hAttributes);

    return $Mockify->getMockObject();
}

sub _createMockedShop {
    my $self = shift;
    my ($IsUnityRegistered) = @_;

    my $PackageName = 'DE_EPAGES::Shop::API::Object::Shop';
    my $hAttributes = {
        IS_UNITY_REGISTERED() => $IsUnityRegistered,
        'LanguageID'          => LANGUAGE_ID_DE,
        PAYMENT_METHODS()     => $self->_createMockedPaymentMethods(),
        'Languages'           => [{'LanguageID' => LANGUAGE_ID_DE}, {'LanguageID' => LANGUAGE_ID_EN}]};

    my $DefaultLanguageId = LANGUAGE_ID_DE;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, $hAttributes, $DefaultLanguageId);
    $Mockify->mock('class', $self->_createMockedShopClass());

    return $Mockify->getMockObject();
}

sub _createMockedPaymentMethods {
    my $self = shift;
    my @PaymentMethods;

    for (my $i = 1; $i <= 2; $i++) {
       push @PaymentMethods, $self->_createMockedPaymentMethodFree(1, $i);
    }

    push @PaymentMethods, $self->_createMockedPaymentMethodFree(0);

    return \@PaymentMethods;
}

sub _createMockedPaymentMethodFree {
    my $self = shift;
    my ($IsNameDefined, $Index) = @_;

    my $PackageName = 'DE_EPAGES::PaymentTypes::API::Object::PaymentMethodFree';
    my $hAttributes = {};
    my $DefaultLanguageId = LANGUAGE_ID_DE;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, $hAttributes, $DefaultLanguageId);
    $Mockify->addLanguageDependentAttribute('Name', {
        LANGUAGE_ID_DE() => $IsNameDefined ? 'Test_' . $Index : undef
    });

    return $Mockify->getMockObject();
}

sub _createMockedShopClass{
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Object::API::Object::Class', [] );
    $Mockify->addMockWithConditionalReturn('existsAttribute', 1, [{'string' => PAYMENT_METHODS}]);
    $Mockify->addMockWithConditionalReturn('existsAttribute', 0, [{'string' => WRONG_PAYMENT_METHODS}]);
    $Mockify->addMockWithConditionalReturn('existsAttribute', 1, [{'string' => IS_UNITY_REGISTERED}]);
    $Mockify->addMockWithConditionalReturn('attribute',
        $self->_createMockedAttribute(1), [{'string' => PAYMENT_METHODS}]);
    $Mockify->addMockWithConditionalReturn('attribute',
        $self->_createMockedAttribute(0), [{'string' => IS_UNITY_REGISTERED}]);
    return $Mockify->getMockObject();
}

sub _createMockedAttribute{
    my $self = shift;
    my ($IsArray) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Object::API::Object::Attribute', [] );
    $Mockify->addMockWithReturnValue('isArray', $IsArray);
    return $Mockify->getMockObject();
}

__PACKAGE__->runScript;
